    processor 6502
    include vcs.h
    include macro.h
    include music.h

; Compile switches

NTSC                    = 0
PAL                     = 1 
COMPILE_VERSION         = NTSC

; Equates

BALLOONFRAMES           = 14
KERNEL_HEIGHT           = $9C
BALLOON_HEIGHT          = 19
PF_LINES                = 26
PF_SIZE                 = 3

; Variables

    SEG.U vars
    ORG $80

caveshape1          ds PF_LINES
caveshape2          ds PF_LINES
caveshape3          ds PF_LINES
caveshape4          ds PF_LINES
verPosBalloon       ds 1
horPosBalloon       ds 1
frameCounter        ds 1
balloonAnim         ds 1
balloonPtr          ds 2
balloonmovPtr       ds 2
balloonOffset       ds 1
PFCounter           ds 1

    SEG     Bank0
    ORG $F800

;#######################
;# MAIN SCREEN DISPLAY #
;#######################

SkipBalloon
                LDA #$00
                SLEEP 7
                BEQ BalloonContinue
SkipBalloon2
                LDA #$00
                SLEEP 7
                BEQ BalloonContinue2

; Start display kernel
Maindisplay
        STA WSYNC           ; Finish current line
        STA VBLANK          ; Stop vertical blank
        STA HMCLR
        LDA #$80
        STA PF0
        LDY #KERNEL_HEIGHT
        LDX #PF_LINES-1
        LDA #$00
        STA PFCounter
load    
        LDA #BALLOON_HEIGHT-1
        DCP balloonOffset
        BCC SkipBalloon
        LDA (balloonmovPtr),Y
        STA HMP0
        LDA (balloonPtr),Y
BalloonContinue
        STA WSYNC
        STA HMOVE
        STA GRP0
        LDA caveshape1,X
        STA PF1
        LDA caveshape2,X
        STA PF2
        DEY
        LDA #BALLOON_HEIGHT-1
        DCP balloonOffset

        SLEEP 12

        LDA caveshape3,X
        STA PF2
        LDA caveshape4,X
        STA PF1
        BCC SkipBalloon2
        LDA (balloonmovPtr),Y
        STA HMP0
        LDA (balloonPtr),Y
BalloonContinue2
        STA WSYNC
        STA HMOVE
        STA GRP0
        LDA #$00
        STA PF1
        STA PF2

        SLEEP 15

        INC PFCounter
        LDA PFCounter
        EOR #3
        BNE PFCounterFine
        STA PFCounter
        DEX
PFCounterFine

        DEY
        BNE  load      ;get ready for next scanline

        STA WSYNC
        STY GRP0
        STY PF0
        STY PF1
        STY PF2

        LDA horPosBalloon
        LDX #$00
        JSR PosElement        
        
        LDX  #$1E
        JMP Overscan

;########################
;# CARTRIDGE MAIN ENTRY #
;########################

Start
    CLEAN_START
    JSR SetStartPos
    LDA #$32           ;
    STA COLUP0         ; Sprite colour
    LDA #$AA
    STA COLUPF         ; Sprite colour
    LDA #$01           ;
    STA CTRLPF

    SET_POINTER balloonPtr, BALLOON18
    SET_POINTER balloonmovPtr, BALLOONM

    LDX #PF_LINES-1
CreateCave
    LDA BALLOONLEVEL0+0*PF_LINES,X
    STA caveshape1,X
    LDA BALLOONLEVEL0+1*PF_LINES,X
    STA caveshape2,X
    LDA BALLOONLEVEL0+2*PF_LINES,X
    STA caveshape3,X
    LDA BALLOONLEVEL0+3*PF_LINES,X
    STA caveshape4,X
    DEX
    BPL CreateCave
    

; Main Program Loop

NEWSCREEN
    VERTICAL_SYNC

; Set VBLANK timer
    IF COMPILE_VERSION = PAL
        LDA #73
    ELSE
        LDA #43
    ENDIF
    STA TIM64T

    LDX balloonAnim
    LDA #KERNEL_HEIGHT
    SEC
    SBC verPosBalloon
    ADC #BALLOON_HEIGHT-1
    STA balloonOffset
    CLC
    ADC balloonAnimTab,X
    STA balloonPtr
    LDA balloonOffset
    CLC
    ADC balloonMoveTab,X
    STA balloonmovPtr

VBLANKLOOP
    LDA INTIM
    BNE VBLANKLOOP  ; WAIT FOR VBLANK TIMER
    JMP Maindisplay

Overscan

; Set Overscan timer
    IF COMPILE_VERSION = PAL
        LDA #65
    ELSE
        LDA #36
    ENDIF
    STA TIM64T

; Collission Detection
    LDA CXP0FB
    ASL
    BCC NoCol
    JSR SetStartPos
NoCol    
    STA CXCLR

; Frame Stuff
    INC frameCounter
    LDA frameCounter
    AND #$01
    BNE NoJoy
    JSR joy        ;overscan. Time to read the joysticks.
NoJoy

    LDA frameCounter
    AND #$07
    BNE OVERSCANLOOP

; Balloon Animation
    INC balloonAnim
    LDA balloonAnim
    EOR #BALLOONFRAMES
    BNE OVERSCANLOOP
    STA balloonAnim

OVERSCANLOOP
    LDA INTIM
    BNE OVERSCANLOOP    ; WAIT FOR OVERSCAN TIMER
    STA WSYNC       ; FINISH WAITING FOR THE CURRENT LINE

    JMP NEWSCREEN

joy     
        LDA SWCHA      
        ASL
        BCS RightDone
        INC horPosBalloon
RightDone
        ASL
        BCS LeftDone
        DEC horPosBalloon
LeftDone
        ASL
        BCS DownDone
        DEC verPosBalloon
DownDone
        ASL
        BCS UpDone
        INC verPosBalloon
UpDone
        RTS

    ORG $FD00
BALLOONLEVEL0
                                .byte   %00011111
                                .byte   %00011111
                                .byte   %00011110
                                .byte   %00011000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00011100
                                .byte   %00011111
                                .byte   %00011111
                                .byte   %00011111
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000

                                .byte   %11111111
                                .byte   %00000111
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00001100
                                .byte   %11111100
                                .byte   %11100000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %10000001
                                .byte   %11000011
                                .byte   %01111100
                                .byte   %00111000
                                .byte   %00010000
                                .byte   %00000001
                                .byte   %00000001
                                .byte   %00000011
                                .byte   %00000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %11100000
                                .byte   %11111000

                                .byte   %10000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000111
                                .byte   %00011111
                                .byte   %11111110
                                .byte   %11111100
                                .byte   %00011000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000010
                                .byte   %00111100
                                .byte   %00111100
                                .byte   %00001100
                                .byte   %00000100
                                .byte   %00000010
                                .byte   %00000000
                                .byte   %10000000
                                .byte   %11000000
                                .byte   %11000000
                                .byte   %11111100
                                .byte   %11111111

                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00000000
                                .byte   %00011111
                                .byte   %00011111
                                .byte   %00011110
                                .byte   %00011100
                                .byte   %00011100
                                .byte   %00011000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00011000
                                .byte   %00011000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00010000
                                .byte   %00011000
                                .byte   %00011110
                                .byte   %00011111

    ORG $FE00

PosElement
       SEC
       STA WSYNC                ;
.waitzone
       SBC #$0F                 ;
       BCS .waitzone            ; RESP loop
       EOR #$07                 ;
       ASL                      ;
       ASL                      ;
       ASL                      ;
       ASL                      ;
       STA HMP0,X
       STA RESP0,X           ; Position it!
       STA WSYNC
       STA HMOVE
       RTS                      ; done, that's all!

SetStartPos
    LDA #$14
    STA verPosBalloon
    LDA #$80
    STA horPosBalloon
    RTS

    ORG $FE80-5

BALLOONLLL
    .byte $00
    .byte $E0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $F0
    .byte $E0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $40
    .byte $40
    .byte $00

BALLOONLL
    .byte $00
    .byte $F0
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $50
    .byte $00

BALLOONL
    .byte $00
    .byte $F0
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $F0
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $40
    .byte $00

BALLOONM
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00

BALLOONR
    .byte $00
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $C0
    .byte $00

BALLOONRR
    .byte $00
    .byte $10
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $10
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $B0
    .byte $00

BALLOONRRR
    .byte $00
    .byte $20
    .byte $10
    .byte $10
    .byte $10
    .byte $10
    .byte $20
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $00
    .byte $C0
    .byte $C0
    .byte $00

    ORG $FF80-5

balloonMoveTab
    .byte <BALLOONLLL - KERNEL_HEIGHT - 1, <BALLOONLLL - KERNEL_HEIGHT - 1
    .byte <BALLOONLL - KERNEL_HEIGHT - 1, <BALLOONL - KERNEL_HEIGHT - 1
    .byte <BALLOONM - KERNEL_HEIGHT - 1, <BALLOONR - KERNEL_HEIGHT - 1
    .byte <BALLOONRR - KERNEL_HEIGHT - 1, <BALLOONRRR - KERNEL_HEIGHT - 1
    .byte <BALLOONRRR - KERNEL_HEIGHT - 1, <BALLOONRR - KERNEL_HEIGHT - 1
    .byte <BALLOONR - KERNEL_HEIGHT - 1, <BALLOONM - KERNEL_HEIGHT - 1
    .byte <BALLOONL - KERNEL_HEIGHT - 1, <BALLOONLL - KERNEL_HEIGHT - 1

balloonAnimTab
    .byte <BALLOON14 - KERNEL_HEIGHT - 1, <BALLOON14 - KERNEL_HEIGHT - 1
    .byte <BALLOON16 - KERNEL_HEIGHT - 1, <BALLOON17 - KERNEL_HEIGHT - 1
    .byte <BALLOON18 - KERNEL_HEIGHT - 1, <BALLOON17 - KERNEL_HEIGHT - 1
    .byte <BALLOON16 - KERNEL_HEIGHT - 1, <BALLOON14 - KERNEL_HEIGHT - 1
    .byte <BALLOON14 - KERNEL_HEIGHT - 1, <BALLOON16 - KERNEL_HEIGHT - 1
    .byte <BALLOON17 - KERNEL_HEIGHT - 1, <BALLOON18 - KERNEL_HEIGHT - 1
    .byte <BALLOON17 - KERNEL_HEIGHT - 1, <BALLOON16 - KERNEL_HEIGHT - 1

BALLOON18
        .byte %00111000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %01111100
        .byte %00111000

BALLOON17
        .byte %00111000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %01111100
        .byte %00111000
        .byte %00000000

BALLOON16
        .byte %00111000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %01111100
        .byte %00111000
        .byte %00000000
        .byte %00000000

BALLOON14
        .byte %00111000
        .byte %00111000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00010000
        .byte %00111000
        .byte %01111100
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %11111110
        .byte %01111100
        .byte %00111000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000

    ORG $FFFA
    .WORD Start
    .WORD Start
    .WORD Start